/*  FreeNote for Sharp SLA300, B500, C7x0, C860 Linux PDA
	Copyright (C) 2003, 2004 Joe Kanemori.<kanemori@ymg.urban.ne.jp>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*
2004/10/17 FreeNote 1.10.0[X
2004/02/12 ver 1.7.1pre
EtHgdl̕ύX
EeLXg̍
EeLXg{bNX̑@\

2004/02/10 ver 1.7.0pre
E

2003/12/22 ver 1.6.1
Ef[^\̌î݁xWFAȉ~ցj

2003/08/17-20 ver 1.3.0
Ef[^̏ki̓_Ȃj

2003/08/05 FreeNote 1.1.1preJ
ENɕԂێ
E`惂[hؑւɏS\
EۑԒZk
EView[h̋ύX
Ej[̌ڂύX
*/
#include "fnpolygon.h"
#include <qcolor.h>
#include <qmessagebox.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qpixmap.h>
#include <qbitmap.h>
#include <qcolor.h>
#include <qimage.h>
#include <qfontdatabase.h>
#include <qpen.h>

const int FONTSIZE[9] = {
	10,
	12,
	16,
	18,
	20,
	24,
	36,
	48,
	72,
};

//FNPolygon
FNPolygon::FNPolygon() : _pen(Qt::black, 1), _points(0), _type(FN_POLYGON)
{
}

FNPolygon::FNPolygon(QPen& p) : _pen(p), _points(0), _type(FN_POLYGON)
{
}

FNPolygon::FNPolygon(const FNPolygon& p)
{
	operator=(p);
}

FNPolygon& FNPolygon::operator=(const FNPolygon& p)
{
	if (this != &p) {
		this->_type = p._type;
		this->_pen = p._pen;
		this->_points = p._points.copy();
	}
	return *this;
}

FNPolygon::~FNPolygon()
{
}

void FNPolygon::setPoints(FNPointList& list)
{
	_points.resize(list.count());
	for (uint i = 0; i < list.count(); ++i) {
		QPoint* p = list.at(i);
		_points.setPoint(i, QPoint(*p));
	}
}

void FNPolygon::translate(int dx, int dy)
{
	_points.translate(dx, dy);
}

void FNPolygon::drawShape(QPainter& pa, bool selected)
{
	if (selected) {
		pa.setPen(QPen(QColor(200, 200, 200), _pen.width()));
	} else {
		pa.setPen(_pen);
	}
	pa.drawPolyline(_points);
	pa.flush();
}

QRect FNPolygon::boundingRect() const
{
	return _points.boundingRect();
}

FNBezier::FNBezier() : FNPolygon()
{
	_type = FN_BEZIER;
}

FNBezier::FNBezier(QPen& p) : FNPolygon(p)
{
	_type = FN_BEZIER;
}

FNBezier::FNBezier(const FNBezier& p) : FNPolygon()
{
	operator=(p);
}

FNBezier& FNBezier::operator=(const FNBezier& p)
{
	if (this != &p) {
		this->_type = p._type;
		this->_pen = p._pen;
		this->_points = p._points.copy();
	}
	return *this;
}

FNBezier::~FNBezier()
{
}

void FNBezier::drawShape(QPainter& pa, bool selected)
{
	if (selected) {
		pa.setPen(QPen(QColor(200, 200, 200),  _pen.width()));
	} else {
		pa.setPen(_pen);
	}
	QPoint tp = _points.point(0);
	for (uint i = 1; i < _points.count(); i += 3) {
		QPointArray p(4);
		p.setPoint(0, tp);
		p.setPoint(1, _points.point(i));
		p.setPoint(2, _points.point(i + 1));
		p.setPoint(3, _points.point(i + 2));
		pa.drawPolyline(p.quadBezier());
		tp = _points.point(i + 2);
	}
	pa.flush();
}

FNEllipse::FNEllipse() : FNPolygon()
{
	_type = FN_ELLIPSE;
}

FNEllipse::FNEllipse(QPen& p) : FNPolygon(p)
{
	_type = FN_ELLIPSE;
}

FNEllipse::FNEllipse(const FNEllipse& p) : FNPolygon()
{
	operator=(p);
}

FNEllipse& FNEllipse::operator=(const FNEllipse& p)
{
	if (this != &p) {
		this->_type = p._type;
		this->_pen = p._pen;
		this->_points = p._points.copy();
	}
	return *this;
}

FNEllipse::~FNEllipse()
{
}

void FNEllipse::drawShape(QPainter& pa, bool selected)
{
	if (selected) {
		pa.setPen(QPen(QColor(200, 200, 200), _pen.width()));
	} else {
		pa.setPen(_pen);
	}
	QPoint sp = _points.point(0);
	QPoint ep = _points.point(1);
	pa.drawEllipse(sp.x(), sp.y(), ep.x() - sp.x(), ep.y() - sp.y());
	pa.flush();
}

FNText::FNText() : FNPolygon()
{
	_type = FN_TEXT;
}

FNText::FNText(QPen& p) : FNPolygon(p)
{
	_type = FN_TEXT;
}

FNText::FNText(const FNText& p) : FNPolygon()
{
	operator=(p);
}

FNText& FNText::operator=(const FNText& p)
{
	if (this != &p) {
		this->_type = p._type;
		this->_pen = p._pen;
		this->_points = p._points.copy();
		this->lines = p.lines;
	}
	return *this;
}

FNText::~FNText()
{
}

void FNText::drawShape(QPainter& pa, bool selected)
{
	static QFontDatabase fbase;
	QFont font = pa.font();
	uint fmax = fbase.pointSizes(font.family()).count() - 1;
	if (selected) {
		pa.setPen(QPen(QColor(200, 200, 200)));
	} else {
		pa.setPen(_pen);
	}
	QPoint sp = _points.point(0);
	//default font size checking...
	if (fmax < _pen.width()) {
		_pen.setWidth(fmax);
	}
	font.setPointSize(FONTSIZE[_pen.width()]);
	QFontMetrics fm(font);
	int h = fm.height();
	int x = sp.x();
	int y = sp.y();
	y += h + 1;
	pa.setFont(font);
	for (uint i = 0; i < lines.count(); ++i) {
		pa.drawText(x + 1, y, lines[i]);
		y = y + h;
	}
	pa.flush();
}
