/*  FreeNote for Sharp SLA300, B500, C7x0, C860 Linux PDA
	Copyright (C) 2003, 2004 Joe Kanemori.<kanemori@ymg.urban.ne.jp>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*
2004/10/17 FreeNote 1.10.0[X
2004/08/26 ver 1.9.3pre
Er@\ǉ

2004/02/16 ver 1.7.3pre
EҏW@\

2004/02/14 ver 1.7.2pre
E@\ǉ

2004/02/12 ver 1.7.1pre
EtHgdl̕ύX
EeLXg̍

2004/02/10 ver 1.7.0pre
E

2003/11/10 ver 1.5.1pre
EȐ`[hǉ

2003/11/09 ver 1.5.0pre
E`[hǉ

2003/09/01-03 ver 1.3.3pre
EXN[̉
EUse all quadrant(Sیgp)j[ǉ

2003/08/31 FreeNote 1.3.2pre
ESXN[

2003/08/23 FreeNote 1.3.0pre
ECȐC

2003/08/15 FreeNote 1.2.1J
Eۑ̃oOC
E_CAO̎
EPNGt@Cւ̏o

2003/08/15 FreeNote 1.2J
EIvVǉ
EXN[KCh
EFreet@C֘At
EACR̕ύX

2003/08/05 FreeNote 1.1.1preJ
ENɕԂێ
E`惂[hؑւɏS\
EۑԒZk
EView[h̋ύX
Ej[̌ڂύX
*/
#ifndef FNCANVAS_H
#define FNCANVAS_H
#include <qpixmap.h>
#include <qwidget.h>
#include <qcolor.h>
#include <qtimer.h>
#include <qfileinfo.h>
#include <qpen.h>
#include <qstringlist.h>
#include <stdio.h>
#include "fnpolygon.h"
#include "fnfinddialog.h"

#define AS_NONE 0
#define AS_HORIZONTAL 1
#define AS_VERTICAL 2
#define AS_BOTH 3

//draw mode
#define MODE_MIN 0
#define MODE_DRAW 0
#define MODE_FORMAT 1
#define MODE_CURVE 2
#define MODE_SMOOTH 3
#define MODE_TEXT 4
#define MODE_ERASE 5
#define MODE_MAX 5
#define PENWIDTH_MAX 8

class Q_EXPORT FNCanvas : public QWidget 
{
	Q_OBJECT
signals:
	void originChanged(int x, int y);
	void resetViewMode();
	void requireText(int x, int y);
public:
	FNCanvas(QWidget* parent = 0, const char* name = 0, WFlags f = 0);
	virtual ~FNCanvas();
	virtual void redraw();
	virtual bool save(const QFileInfo& info);
	virtual bool load(const QFileInfo& info);
	virtual bool import(const QFileInfo& info);
	virtual void open(FNPolygonList& list, FILE* fp);
	virtual bool exportPNG(const QFileInfo& info, QPixmap& buf);
	virtual void setScrollMode(int as);
	virtual void setGuide(bool f);
	virtual QPoint getTopLeft();
	virtual void rebuild();
	const QPoint& origin() const {return _origin;};

    void setScrollTiming(int v);
    void setVStep(int v);
    void setHStep(int v);
    void setSEraser(int v);
    void setLEraser(int v);
    void setMargin(int v);
    void setFontName(const QString& s) {fontname = s;};
    const QPixmap& wbuffer() const {
    	return _buffer;
    }
    void setShowRuler(bool f) {
    	_showRuler = f;
   		redraw();
    }
    void setUseAllQuadrant(bool f) {
    	if (_isUseAllQuadrant) {
			QPoint o = getTopLeft();
			rebuild();
			setOrigin(-o.x(), -o.y());
    	}
    	_isUseAllQuadrant = f;
   		redraw();
    }
    bool isSelected() {return _isSelected;};
    void selectionMoveTo(int dx, int dy);
public slots:
	virtual void setOrigin(QPoint& o);
	virtual void setOrigin(int ox, int oy);
	virtual void resetOrigin();
	virtual void setPensize(int sz);
	virtual void modeChanged(int mode);
	virtual void viewChanged(bool f);
	virtual void changeColor(QRgb c);
	virtual void find();
	virtual void undo();
	virtual void redo();
	virtual void clear();
	virtual void erase();
	virtual void CR();
	virtual void autoScroll();
	virtual void textEdit(int x, int y, FNText* obj = NULL);
	virtual void copy();
	virtual void paste();
protected:
	virtual void mousePressEvent(QMouseEvent* evt);
	virtual void mouseMoveEvent(QMouseEvent* evt);
	virtual void mouseReleaseEvent(QMouseEvent* evt);
	virtual void paintEvent(QPaintEvent*);
	virtual void resizeEvent(QResizeEvent* evt);
	virtual void clearList(FNPolygonList& list);
	virtual void drawRect(QPainter& pa, const QRect& r);
	virtual void resetSelection();
private:
	void redobuf_flush();
	QRect getMatrix(const QRect& r) const;
	QPoint* createPts(int x, int y);
	FNPolygon* createPolygon(QPen& pen);
	FNPolygon* createBezier(QPen& pen);
	FNPolygon* createEllipse(QPen& pen);
	FNPolygon* createText(QPen& pen, QStringList& lines);
	

	QPixmap _buffer;
	QPoint _last;
	int _tdx;
	int _tdy;
	QRect _selection;
	QRect _preRect;
	QPoint _origin;
	QTimer* _timer;

	FNPointList _tracks;
	FNPolygonList _draws;
	FNPolygonList _marks;
	FNPolygonList _selected;
	FNPolygonList _undobuf;
	FNPolygonList _clipboard;

	FNText* _txtTmp;
	QPen _pen;
	int _asMode;
	int _drawMode;
	uint _disp_offset;
	int _eraser_l;
	int _eraser_s;
	int _h_step;
	int _v_step;
	int _margin;
	int _scrollTiming;

	bool _viewMode;
	bool _isWaiting;
	bool _isDragging;
	bool _isDrawing;
	bool _isHeadingEnables;
	bool _isShowGuide;
	bool _isUseAllQuadrant;
	bool _showRuler;
	bool _isSelected;
	
	float _scale_x;
	float _scale_y;
	
	QString fontname;

	FNFindDialog* _dlgFind;
};
#endif //FNCANVAS_H
